using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

static class Program
{ static void Main() { Game1 game = new Game1(); game.Run(); }

public class Game1 : Microsoft.Xna.Framework.Game
{ private GraphicsDeviceManager g;
  private VertexBuffer vbuffer;  
  private BasicEffect  effect;
  private EffectPass   pass;
  private float rotAngle = 0f;
  private const int xsteps = 100;
  
  public Game1()
  { Window.Title = "Light1";
    Window.AllowUserResizing = true;
    g = new GraphicsDeviceManager( this );
    g.IsFullScreen = false;
    g.PreferredBackBufferWidth  = 600;
    g.PreferredBackBufferHeight = 600;
  }
  protected override void Initialize()
  { VertexPositionNormalTexture[] v = new VertexPositionNormalTexture[xsteps];
    float  amplitude = 0.2f;
    double frequency = 4.0 * MathHelper.TwoPi;
    for ( int i = 0; i < xsteps; i++ )
    { float x = (float)i / xsteps;
      v[i].Position.X          = -0.5f + x;
      v[i].TextureCoordinate.X =         x;
      if ( i%2 == 0 ) { v[i].Position.Y =  0.5f; v[i].TextureCoordinate.Y = 0f; }
      else            { v[i].Position.Y = -0.5f; v[i].TextureCoordinate.Y = 1f; }
      v[i].Position.Z = amplitude * (float)System.Math.Sin( x*frequency );
    }
    for ( int i = 0; i < xsteps-2; i++ )
    { Vector3 a = v[i+1].Position - v[i].Position;
      Vector3 b = v[i+2].Position - v[i].Position;
      Vector3 c = Vector3.Cross( a, b );
      c.Normalize();
      if ( c.Z > 0f ) v[i].Normal = c;
      else            v[i].Normal = Vector3.Negate( c );
    }
    vbuffer = new VertexBuffer( g.GraphicsDevice, typeof(VertexPositionNormalTexture), xsteps, BufferUsage.WriteOnly );
    vbuffer.SetData< VertexPositionNormalTexture >( v );
    effect = new BasicEffect( g.GraphicsDevice, null );
    effect.View = Matrix.CreateLookAt( new Vector3(0, 0, 4), Vector3.Zero, Vector3.Up );
    effect.Projection = Matrix.CreatePerspectiveFieldOfView( MathHelper.Pi/8, 1f, 1f, 10f );
    effect.LightingEnabled = true;
    effect.Texture = new ContentManager(Services).Load< Texture2D >("Content\\Foto");
    effect.TextureEnabled = true;
    pass = effect.CurrentTechnique.Passes[0];
    BasicDirectionalLight light = effect.DirectionalLight0;
    light.Direction = new Vector3( 0f, 0f, -1f ); 
    light.Direction.Normalize();
    light.DiffuseColor  = Color.White.ToVector3();
    light.SpecularColor = Color.CornflowerBlue.ToVector3();
    light.Enabled = true;
  }
  protected override void Update( GameTime gameTime )
  { g.GraphicsDevice.Vertices[0].SetSource( vbuffer, 0, VertexPositionNormalTexture.SizeInBytes );
    g.GraphicsDevice.VertexDeclaration = new VertexDeclaration( g.GraphicsDevice, VertexPositionNormalTexture.VertexElements );
    g.GraphicsDevice.RenderState.CullMode = CullMode.None;
    //g.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
    g.GraphicsDevice.RenderState.DepthBufferEnable      = false;
    g.GraphicsDevice.RenderState.DepthBufferWriteEnable = false;
  }
  protected override void Draw( GameTime gameTime )
  { g.GraphicsDevice.Clear( Color.DarkBlue );
    //effect.World = Matrix.CreateRotationZ( rotAngle += 0.01f );
    effect.World = Matrix.CreateRotationY( rotAngle += 0.01f );
    //effect.World = Matrix.CreateRotationX( rotAngle += 0.01f );
    effect.Begin();
      pass.Begin();
        try { g.GraphicsDevice.DrawPrimitives( PrimitiveType.TriangleStrip, 0, xsteps-2 ); } catch {}
      pass.End();
    effect.End();
    base.Draw(gameTime);
  }
} // end of class Game1
} // end of class Program